﻿' (C) Copyright 2010 by Autodesk, Inc. 
'

'名前空間のインポート
Imports System
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.EditorInput

'これは必修ではありません。しかし、ロードのパーフォーマンス向上させます。
' This line is not mandatory, but improves loading performances
<Assembly: CommandClass(GetType(Lab1.MyCommands))> 

Namespace Lab1

    ' This class is instantiated by AutoCAD for each document when
    ' a command is called by the user the first time in the context
    ' of a given document. In other words, non static data in this class
    ' is implicitly per-document!
    Public Class MyCommands

        'CreateCircleコマンド
        '円をデータベースに追加
        <CommandMethod("CreateCircle")> _
        Public Sub Lab1()

            'カレント図面のデータベースを取得 
            'コメントアウトしてある方法でも可能
            Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
            ' Dim db As Database = HostApplicationServices.WorkingDatabase()

            'エディタオブジェクトを取得
            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            ' トランザクションを開始
            Dim trans As Transaction = db.TransactionManager.StartTransaction

            Try
                ' 円オブジェクトをメモリ上に作成
                Dim center As Point3d = New Point3d(0, 0, 0) ' 中心点
                Dim normal As Vector3d = New Vector3d(0, 0, 1) ' 法線ベクトル
                Dim circle As Circle = New Circle(center, normal, 20) '円オブジェクトを作成

                ' ブロックテーブルを読み込み専用モードでオープン
                Dim bt As BlockTable = trans.GetObject(db.BlockTableId, OpenMode.ForRead)

                ' モデル空間を下書き込みモードでオープン
                Dim btr As BlockTableRecord = trans.GetObject(bt.Item(BlockTableRecord.ModelSpace), OpenMode.ForWrite)

                ' 円を図面データベースに追加
                btr.AppendEntity(circle)

                ' トランザクションに円を図面データベースに追加を通知
                trans.AddNewlyCreatedDBObject(circle, True)

                ' トランザクションをコミット
                trans.Commit()


            Catch ex As System.Exception

                ' 例外（エラー）発生して場合は例外とコマンドプロンプトに出力
                ed.WriteMessage(ex.ToString)

            Finally
                ' トランザクションを破棄
                ' コミットしないで破棄した場合は編集結果は図面に反映されない
                trans.Dispose()

            End Try

        End Sub

        ' ChangeColor コマンド
        ' 図形を選択して色を変更する
        <CommandMethod("ChangeColor")> _
        Public Sub Lab1_2()

            'カレント図面のデータベースを取得 
            'コメントアウトしてある方法でも可能
            Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
            ' Dim db As Database = HostApplicationServices.WorkingDatabase()

            'エディタオブジェクトを取得
            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            ' トランザクションを開始
            Dim trans As Transaction = db.TransactionManager.StartTransaction

            Try
                '図形（エンティティ）を選択するプロンプトを作成
                Dim prEnt As PromptEntityOptions = New PromptEntityOptions("色を変える図形を選択: ")

                '図形を選択
                Dim prEntRes As PromptEntityResult = ed.GetEntity(prEnt)

                '選択が正常終了したか確認
                If prEntRes.Status <> PromptStatus.OK Then
                    Return
                End If

                '選択された図形(エンティティ）を書き込みモードでオープン
                Dim entity As Entity = trans.GetObject(prEntRes.ObjectId, OpenMode.ForWrite)

                'AutoCAD Color Index (ACI)の値に１を加えて図形の色を変更
                entity.ColorIndex = entity.ColorIndex + 1

                'トランザクションをコミット
                trans.Commit()

            Catch ex As System.Exception

                ' 例外（エラー）発生して場合は例外とコマンドプロンプトに出力
                ed.WriteMessage(ex.ToString)

            Finally

                ' トランザクションを破棄
                ' コミットしないで破棄した場合は編集結果は図面に反映されない
                trans.Dispose()

            End Try
        End Sub

        ' AddMyDataコマンド
        ' 名前付きオブジェクトディクショナリにアプリケーション固有のデータを保存
        <CommandMethod("AddMyData")> _
        Public Sub Lab1_3()
            'カレント図面のデータベースを取得 
            'コメントアウトしてある方法でも可能
            Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
            ' Dim db As Database = HostApplicationServices.WorkingDatabase()

            'エディタオブジェクトを取得
            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            ' トランザクションを開始
            Dim trans As Transaction = db.TransactionManager.StartTransaction

            Try
                ' 名前付きオブジェクトディクショナリを書き込みモードでオープン
                Dim NOD As DBDictionary = trans.GetObject(db.NamedObjectsDictionaryId, OpenMode.ForWrite)

                'ディクショナリ作成して"MyData"という名前で名前付きオブジェクトディクショナリに追加
                Dim myDict As DBDictionary = New DBDictionary
                NOD.SetAt("MyData", myDict)
                trans.AddNewlyCreatedDBObject(myDict, True)

                '追加した"MyData"というディクショナリを書き込みモードでオープン
                Dim myDict2 As DBDictionary
                myDict2 = trans.GetObject(NOD.GetAt("MyData"), OpenMode.ForWrite)

                'Xrecordオブジェクトに文字列と整数値を追加

                Dim xRec As New Xrecord
                xRec.Data = New ResultBuffer(New TypedValue(DxfCode.Text, "Taro Yamada"), _
                                             New TypedValue(DxfCode.Int16, 35))

                '"NameAndAge"という名前でXrecordオブジェクトをディクショナリに追加
                myDict2.SetAt("NameAndAge", xRec)
                trans.AddNewlyCreatedDBObject(xRec, True)

                'トランザクションをコミット
                trans.Commit()

            Catch ex As System.Exception

                ' 例外（エラー）発生して場合は例外とコマンドプロンプトに出力
                ed.WriteMessage(ex.ToString)

            Finally

                ' トランザクションを破棄
                ' コミットしないで破棄した場合は編集結果は図面に反映されない
                trans.Dispose()

            End Try
        End Sub

        ' AddEventこまんど
        ' コマンド終了の監視を開始します
        <CommandMethod("AddEvent")> _
        Public Sub addEvent()
            Dim doc As Document = Application.DocumentManager.MdiActiveDocument

            'CommandEventHandlerデリゲートでイベントハンドラ関数とイベントをひもづけします
            AddHandler doc.CommandEnded, New CommandEventHandler(AddressOf cmdEnded)

        End Sub

        'イベントハンドラ関数
        Public Sub cmdEnded(ByVal sender As Object, ByVal e As CommandEventArgs)
            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            '終了してコマンド名を表示します
            ed.WriteMessage(ControlChars.Lf + "コマンドが終了: " + e.GlobalCommandName.ToString)

        End Sub

        'RemoveEventコマンド
        'イベントの監視を終了します
        <CommandMethod("RemoveEvent")> _
        Public Sub removeEvent()
            Dim doc As Document = Application.DocumentManager.MdiActiveDocument

            'イベントハンドラをイベントから登録解除します
            RemoveHandler doc.CommandEnded, AddressOf cmdEnded

        End Sub

        '以下はCommandMethod属性をしたコマンド定義の例です
        '
        ' The CommandMethod attribute can be applied to any public  member 
        ' function of any public class.
        ' The function should take no arguments and return nothing.
        ' If the method is an instance member then the enclosing class is 
        ' instantiated for each document. If the member is a static member then
        ' the enclosing class is NOT instantiated.
        '
        ' NOTE: CommandMethod has overloads where you can provide helpid and
        ' context menu.

        'モーダルコマンドとローカライズ名のコマンド定義の例
        'AutoCADは"MyCommandLocal"というIDのリソース文字列を探します。
        '見つからない場合は"MyCommandLocalを文字列をしてローカルコマンド名とします。
        'リソースはソリューションエクスポーラのmyCommands.vbの下のmyCommands.resxを
        'クリックしてください
        '
        ' Modal Command with localized name
        ' AutoCAD will search for a resource string with Id "MyCommandLocal" in the 
        ' same namespace as this command class. 
        ' If a resource string is not found, then the string "MyLocalCommand" is used 
        ' as the localized command name.
        ' To view/edit the resx file defining the resource strings for this command, 
        ' * click the 'Show All Files' button in the Solution Explorer;
        ' * expand the tree node for myCommands.vb;
        ' * and double click on myCommands.resx
        <CommandMethod("MyGroup", "MyCommand", "MyCommandLocal", CommandFlags.Modal)> _
        Public Sub MyCommand() ' This method can have any name
            ' Put your command code here
        End Sub

        'モーダルコマンドでピックファースト選択をするために必要なコマンドの定義
        ' Modal Command with pickfirst selection
        <CommandMethod("MyGroup", "MyPickFirst", "MyPickFirstLocal", CommandFlags.Modal + CommandFlags.UsePickSet)> _
        Public Sub MyPickFirst() ' This method can have any name
            Dim result As PromptSelectionResult = Application.DocumentManager.MdiActiveDocument.Editor.GetSelection()
            If (result.Status = PromptStatus.OK) Then
                ' There are selected entities
                ' Put your command using pickfirst set code here
            Else
                ' There are no selected entities
                ' Put your command code here
            End If
        End Sub


        'ローカルコマンド名を持ち、アプリケーション実行コンテキストで実行されるコマンドの定義
        ' Application Session Command with localized name
        <CommandMethod("MyGroup", "MySessionCmd", "MySessionCmdLocal", CommandFlags.Modal + CommandFlags.Session)> _
        Public Sub MySessionCmd() ' This method can have any name
            ' Put your command code here
        End Sub

        'AutoCAD .NETでLISPから呼び出せる関数の定義
        '数多くの戻り値がサポートされています。
        ' LispFunction is similar to CommandMethod but it creates a lisp 
        ' callable function. Many return types are supported not just string
        ' or integer.
        <LispFunction("MyLispFunction", "MyLispFunctionLocal")> _
        Public Function MyLispFunction(ByVal args As ResultBuffer) ' This method can have any name
            ' Put your command code here

            ' Return a value to the AutoCAD Lisp Interpreter
            Return 1
        End Function

    End Class

End Namespace